"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("tns-core-modules/utils/utils");
var application = require("tns-core-modules/application");
var mlkit_cameraview_common_1 = require("./mlkit-cameraview-common");
// TODO pause/resume handling
var MLKitCameraView = /** @class */ (function (_super) {
    __extends(MLKitCameraView, _super);
    function MLKitCameraView() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    MLKitCameraView.prototype.disposeNativeView = function () {
        _super.prototype.disposeNativeView.call(this);
        if (this.captureSession) {
            this.captureSession.stopRunning();
            this.captureSession = undefined;
        }
        this.captureDevice = undefined;
        this.previewLayer = undefined;
        this.cameraView = undefined;
        application.off("orientationChanged");
    };
    MLKitCameraView.prototype.createNativeView = function () {
        var v = _super.prototype.createNativeView.call(this);
        if (this.canUseCamera()) {
            this.initView();
        }
        else {
            console.log("There's no Camera on this device :(");
        }
        return v;
    };
    MLKitCameraView.prototype.canUseCamera = function () {
        // TODO also check for availability of AVCaptureDeviceDiscoverySession (iOS 10)
        return AVCaptureDevice.defaultDeviceWithMediaType(AVMediaTypeVideo) !== null;
    };
    MLKitCameraView.prototype.initView = function () {
        // if (this.preferFrontCamera) {
        // this._reader.switchDeviceInput();
        // }
        // find a suitable device
        this.captureDevice = AVCaptureDeviceDiscoverySession.discoverySessionWithDeviceTypesMediaTypePosition([AVCaptureDeviceTypeBuiltInWideAngleCamera], AVMediaTypeVideo, 1 /* Back */).devices.firstObject;
        // begin the session
        this.captureSession = AVCaptureSession.new();
        this.captureSession.sessionPreset = AVCaptureSessionPreset1280x720;
        var captureDeviceInput = AVCaptureDeviceInput.deviceInputWithDeviceError(this.captureDevice);
        this.captureSession.addInput(captureDeviceInput);
        this.previewLayer = AVCaptureVideoPreviewLayer.layerWithSession(this.captureSession);
        this.previewLayer.videoGravity = AVLayerVideoGravityResizeAspectFill;
        // TODO use connection.videoOrientation (as this one is deprecated)
        if (utils_1.ios.isLandscape()) {
            this.previewLayer.orientation = 3 /* LandscapeRight */;
        }
        else {
            this.previewLayer.orientation = 1 /* Portrait */;
        }
        // note that when rotating back to portrait, this event fires very late.. not much we can do I think
        application.off("orientationChanged");
        application.on("orientationChanged", this.rotateOnOrientationChange.bind(this));
        if (this.ios) {
            this.ios.layer.addSublayer(this.previewLayer);
        }
        this.captureSession.startRunning();
        this.cameraView = TNSMLKitCameraView.alloc().initWithCaptureSession(this.captureSession);
        this.cameraView.processEveryXFrames = this.processEveryNthFrame;
        // this orientation is how the captured image is rotated (and shown)
        // TODO do this for textrecognition, but NOT for barcode scanning (so set in subclass)
        if (this.rotateRecording()) {
            this.cameraView.imageOrientation = 3 /* Right */;
        }
        this.cameraView.delegate = TNSMLKitCameraViewDelegateImpl.createWithOwnerResultCallbackAndOptions(new WeakRef(this), function (data) { }, {});
    };
    MLKitCameraView.prototype.rotateOnOrientationChange = function (args) {
        console.log(">>> rotateOnOrientationChange");
        if (this.previewLayer) {
            if (args.newValue === "landscape") {
                // with both set to 1, detection works in both portr and landsc (but the image is on its side in landscape)
                this.previewLayer.connection.videoOrientation = 3 /* LandscapeRight */; // deviceOrientation === UIDeviceOrientation.LandscapeLeft ? AVCaptureVideoOrientation.LandscapeRight : AVCaptureVideoOrientation.LandscapeLeft;
            }
            else if (args.newValue === "portrait") {
                this.previewLayer.connection.videoOrientation = 1 /* Portrait */;
            }
        }
    };
    MLKitCameraView.prototype.onLayout = function (left, top, right, bottom) {
        _super.prototype.onLayout.call(this, left, top, right, bottom);
        if (this.ios && this.canUseCamera()) {
            this.previewLayer.frame = this.ios.layer.bounds;
            // this.previewLayer.bounds = this.ios.layer.bounds;
        }
    };
    return MLKitCameraView;
}(mlkit_cameraview_common_1.MLKitCameraView));
exports.MLKitCameraView = MLKitCameraView;
var TNSMLKitCameraViewDelegateImpl = /** @class */ (function (_super) {
    __extends(TNSMLKitCameraViewDelegateImpl, _super);
    function TNSMLKitCameraViewDelegateImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    TNSMLKitCameraViewDelegateImpl.createWithOwnerResultCallbackAndOptions = function (owner, callback, options) {
        var delegate = TNSMLKitCameraViewDelegateImpl.new();
        delegate.owner = owner;
        delegate.options = options;
        delegate.resultCallback = callback;
        delegate.detector = owner.get().createDetector();
        delegate.onSuccessListener = owner.get().createSuccessListener();
        delegate.rotateRecording = owner.get().rotateRecording();
        return delegate;
    };
    TNSMLKitCameraViewDelegateImpl.prototype.cameraDidOutputImage = function (image) {
        if (image) {
            // more fine grained
            // console.log("UIDevice.current.orientation: " + UIDevice.currentDevice.orientation);
            var fIRVisionImage = FIRVisionImage.alloc().initWithImage(image);
            if (this.rotateRecording && !utils_1.ios.isLandscape()) {
                var fIRVisionImageMetadata = FIRVisionImageMetadata.new();
                // TODO see detectorOrientationFrom in MLKitExample
                fIRVisionImageMetadata.orientation = 6 /* RightTop */;
                fIRVisionImage.metadata = fIRVisionImageMetadata;
            }
            console.log(">>> delegate.detector: " + this.detector);
            this.detector.detectInImageCompletion(fIRVisionImage, this.onSuccessListener);
            // TODO remove (this is for debugging only)... but we could pass the image to the UI anyway (although the rotation may need to be corrected in some cases)
            this.owner.get().notify({
                eventName: "scanResultImage",
                object: this.owner.get(),
                value: image
            });
        }
    };
    TNSMLKitCameraViewDelegateImpl.ObjCProtocols = [TNSMLKitCameraViewDelegate];
    return TNSMLKitCameraViewDelegateImpl;
}(NSObject));
